# -*- coding: utf-8 -*-
# Copyright (c) 2013-2023 The tektronix developers. All rights reserved.
# Project site: https://github.com/questrail/tektronix
# Use of this source code is governed by a MIT-style license that
# can be found in the LICENSE.txt file for the project.
"""Package initialization.
"""
__version__ = "0.1.1"



# -*- coding: utf-8 -*-
# Copyright (c) 2013-2023 The tektronix developers. All rights reserved.
# Project site: https://github.com/questrail/tektronix
# Use of this source code is governed by a MIT-style license that
# can be found in the LICENSE.txt file for the project.
"""Read a CSV file saved by a RSA500 Spectrum Analyzer
"""

# Standard module imports
import csv
import os
import sys

# Data analysis related imports
import numpy as np


def read_csv_file(filename):
    """Read csv file into a numpy array"""
    header = {}
    infile = open(filename, "r", newline="", encoding="utf8")

    with infile as csvfile:
        reader = csv.reader(
            (line.replace("\0", "") for line in csvfile), delimiter=","
        )
        header["file"] = os.path.basename(filename)
        temp_row = next(reader)
        if temp_row[0] != "Spectrum":
            sys.exit(1)
        header["timestamp"] = temp_row[1]
        temp_row = next(reader)
        temp_row = next(reader)
        header["center_freq"] = float(temp_row[1])
        header["center_freq_units"] = temp_row[2]
        temp_row = next(reader)
        header["ref_level"] = float(temp_row[1])
        header["ref_level_units"] = temp_row[2]

        # Read until we find the [Parameters] section.
        while True:
            temp_row = next(reader)
            if len(temp_row) == 1 and temp_row[0] == "[Parameters]":
                break

        temp_row = next(reader)
        header["span_freq"] = float(temp_row[1])
        header["span_freq_units"] = temp_row[2]
        temp_row = next(reader)
        header["resolution_bw"] = float(temp_row[1])
        header["resolution_bw_units"] = temp_row[2]
        temp_row = next(reader)
        header["rbw_window_type"] = temp_row[1]

        # Read until we find the [Traces] section.
        while True:
            temp_row = next(reader)
            if len(temp_row) == 1 and temp_row[0] == "[Traces]":
                break
        temp_row = next(reader)
        temp_row = next(reader)
        temp_row = next(reader)
        num_points = int(temp_row[1])
        header["num_points"] = num_points
        # FIXME(mdr): I need an example file with multiple traces to determine
        # how to parse more than just a single trace.
        num_traces = 1
        header["num_traces"] = num_traces
        temp_row = next(reader)
        header["frequency_unit"] = temp_row[2]
        temp_row = next(reader)

        data_array = []

        for _ in range(num_points):
            temp_row = next(reader)
            data_array.append((float(temp_row[1]), float(temp_row[0])))
            data = np.array(
                data_array,
                dtype={
                    "names": ("frequency", "amplitude"),
                    "formats": ("f8", "f8"),
                },
            )

    return (header, data)



# -*- coding: utf-8 -*-
# Copyright (c) 2013-2022 The tektronix developers. All rights reserved.
# Project site: https://github.com/questrail/tektronix
# Use of this source code is governed by a MIT-style license that
# can be found in the LICENSE.txt file for the project.
"""Unit tests for tektronix/rsa500.py.
"""
import unittest
import logging

import numpy.testing as npt

from unipath import Path

from tektronix import rsa500

TEST_DIR = Path(__file__).ancestor(1)


# Setup logging
logging.basicConfig(
    level=logging.DEBUG, format=" %(asctime)s -  %(levelname)s - %(message)s"
)


class TestReadingCSVFiles(unittest.TestCase):
    def setUp(self):
        test_csv_file = Path(TEST_DIR, "sample_data", "1M-11M.csv")
        print(test_csv_file)
        (self.header, self.data) = rsa500.read_csv_file(test_csv_file)

    def test_header_when_reading_csv_file(self):
        self.assertEqual(self.header["file"], "1M-11M.csv")
        self.assertEqual(self.header["center_freq"], 6_000_000.0)
        self.assertEqual(self.header["span_freq"], 10_000_000.0)
        self.assertEqual(self.header["resolution_bw"], 9_000.0)
        self.assertEqual(self.header["resolution_bw_units"], "Hz")
        self.assertEqual(self.header["num_traces"], 1)
        self.assertEqual(self.header["num_points"], 801)

    def test_data_when_reading_csv_file(self):
        self.assertEqual(self.data.shape, (801,))
        self.assertEqual(self.data["amplitude"].shape, (801,))
        freq_data = [
            (self.data["frequency"][0], 1_000_000.0),
            (self.data["frequency"][1], 1_012_500.0),
            (self.data["frequency"][799], 10_987_500.0),
            (self.data["frequency"][800], 11_000_000.0),
            (self.data["frequency"][-2], 10_987_500.0),
            (self.data["frequency"][-1], 11_000_000.0),
        ]
        for actual, desired in freq_data:
            npt.assert_almost_equal(actual, desired)
        amp_data = [
            (self.data["amplitude"][0], 57.427009582519531),
            (self.data["amplitude"][1], 40.868099212646484),
            (self.data["amplitude"][-2], 17.863529205322266),
            (self.data["amplitude"][-1], 20.075450897216797),
        ]
        for actual, desired in amp_data:
            npt.assert_almost_equal(actual, desired)


if __name__ == "__main__":
    unittest.main()



# -*- coding: utf-8 -*-
# Copyright (c) 2013-2022 The tektronix developers. All rights reserved.
# Project site: https://github.com/questrail/tektronix
# Use of this source code is governed by a MIT-style license that
# can be found in the LICENSE.txt file for the project.
"""Dynamic setup file.
"""

# Standard module imports
import codecs
import os
import re

import setuptools

with open("README.md", "r", encoding="utf-8") as fh:
    long_description = fh.read()

here = os.path.abspath(os.path.dirname(__file__))


def read(*parts):
    """Read parts of a file

    Taken from pip's setup.py
    intentionally *not* adding an encoding option to open
    see: https://github.com/pypa/virtualenv/issues/201#issuecomment-3145690
    """
    return codecs.open(os.path.join(here, *parts), "r").read()


def find_version(*file_paths):
    """Find version in source file

    Read the version number from a source file.
    Code taken from pip's setup.py
    """
    version_file = read(*file_paths)
    # The version line must have the form:
    # __version__ = 'ver'
    version_match = re.search(
        r"^__version__ = ['\"]([^'\"]*)['\"]", version_file, re.M
    )
    if version_match:
        return version_match.group(1)
    raise RuntimeError("Unable to find version string.")


setuptools.setup(
    name="tektronix",
    version=find_version("src", "tektronix", "__init__.py"),
    author="Matthew Rankin",
    author_email="matthew@questrail.com",
    description="Package for working with Keysight/Agilent/HP test equipment",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="http://github.com/questrail/tektronix",
    project_urls={
        "Bug Tracker": "https://github.com/questrail/tektronix/issues",
    },
    package_dir={"": "src"},
    packages=setuptools.find_packages(where="src"),
    python_requires=">=3.6",
    license="MIT",
    requires=["numpy (>=1.6.0)"],
    classifiers=[
        "Programming Language :: Python",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.4",
        "Programming Language :: Python :: 3.5",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9",
        "License :: OSI Approved :: MIT License",
        "Development Status :: 5 - Production/Stable",
        "Operating System :: OS Independent",
        "Intended Audience :: Developers",
        "Topic :: Software Development :: Libraries :: Python Modules",
    ],
)
